<?php
namespace Teknisa\PLA\Modules\Rfid\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

class Plaparamrfid extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "pla_paramrfid";
    protected $dataSourceNameToSave = "pla_paramrfid_to_save";
    protected $crudService;
    protected $parameterBagColumns = array(
        "NRORG",
        "CDFILIAL",
        "CDSERVFILI",
        "IDTPCONTAGEM",
        "CDLEITOR",
        "CDANTENA",
        "CDGRUPORFID",
        "HRINILEITURA",
        "HRFINLEITURA"
    );

    public function __construct($crudService, $parameterBag ,$dataSourceManager, $environment ) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
        
    }

    function retornaMaxNrparamrfid(DTO\Request $request, DTO\Response $response){
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->setDataSourceName("pla_paramrfid_max_nrparamrfid");
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $response->addDataSet($dataSet);
    }

    function retornaConflitoHorario(DTO\Request $request, DTO\Response $response){
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->setDataSourceName("pla_paramrfid_conflito_horario");
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        $response->addDataSet($dataSet);
    }


     public function save(DTO\Request $request, DTO\Response $response) {
        $rowToSave = $request->getDataSet()->getRows()[0];
        
        $conflitoFilialResultSet = $this->getConflitoLeitor($rowToSave);
        if(count($conflitoFilialResultSet->getRows()) != 0){
            $response->setError(new Error('Este leitor e antena já estão cadastrados para unidade: '.$conflitoFilialResultSet->getRows()[0]['NMFILIAL'],500));
            return;
        } 
        $conflitoHorarioResultSet = $this->getConflitoLeitor($rowToSave);
        if(count($conflitoHorarioResultSet->getRows()) != 0){
            $response->setError(new Error('Existe conflito de horário com o serviço: '.$conflitoHorarioResultSet->getRows()[0]['NMSERVICO'],500));
            return;
        }
        $rowToSave['HRFINLEITURA'] = str_replace(':','', $rowToSave['HRFINLEITURA']);
        $rowToSave['HRINILEITURA'] = str_replace(':','', $rowToSave['HRINILEITURA']);
        $rowToSave['HRCONTAGEM'] = str_replace(':','', $rowToSave['HRCONTAGEM']);
        parent::save($request, $response);
        
    }
    
    private function getConflitoHorario($row){
        $conditions = array(
            array(
                'columnName' => 'HRFINLEITURA',
                'operator' => '=',
                'value' => $row['HRFINLEITURA'],
            ),
            array(
                'columnName' => 'HRINILEITURA',
                'operator' => '=',
                'value' => $row['HRINILEITURA'],
            ),
            array(
                'columnName' => 'IDTPCONTAGEM',
                'operator' => '=',
                'value' => $row['IDTPCONTAGEM'],
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->environment->getNrOrgTrab(),
            ),
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL'],
            ),
            array(
                'columnName' => 'CDANTENA',
                'operator' => '=',
                'value' => $row['CDANTENA'],
            ),array(
                'columnName' => 'CDLEITOR',
                'operator' => '=',
                'value' => $row['CDLEITOR'],
            )
        );
        $filterCriteria = new FilterCriteria("pla_paramrfid_conflito_horario", $conditions, 1, 1);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet;
    }
    
    private function getConflitoLeitor($row){
         $conditions = array(
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->environment->getNrOrgTrab(),
            ),
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $row['CDFILIAL'],
            ),
            array(
                'columnName' => 'CDANTENA',
                'operator' => '=',
                'value' => $row['CDANTENA'],
            ),array(
                'columnName' => 'CDLEITOR',
                'operator' => '=',
                'value' => $row['CDLEITOR'],
            )
        );
        $filterCriteria = new FilterCriteria("pla_paramrfid_conflito_leitor_antena", $conditions, 1, 1);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, ["NRORG","CDFILIAL","CDANTENA","CDLEITOR"]);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet;
    }
} 