<?php

namespace Teknisa\PLA\Modules\Servico\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\DBAL\Connection;
use Exception;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ger04102Manutencao extends \Teknisa\GER\Controllers\TeknisaCrud {
	
	protected $connection;
	protected $environment;
	protected $newCode;
	protected $crudService;
	protected $servicoService;

    public function __construct($connection, $environment, $newCode, $crudService, $servicoService){
        $this->connection   = $connection;
        $this->environment  = $environment;
        $this->newCode      = $newCode;
        $this->crudService  = $crudService;
        $this->servicoService = $servicoService;
	}
	
    public function filtraGrid($request, $response){
    	$conditions = $request->getFilterCriteria()->getConditions();
    	$filterServ = array_slice($conditions, 1, 5);
    	$filterGrid = array_slice($conditions, 6);
    	$servicos = $this->retornaListaServico($filterServ);
    	$nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrgTrab());
    	foreach($servicos as $servico){
    	    $this->insertLista($nrlista, $servico['CDFILIAL'].$servico['CDSERVFILI']);
    	}
    	array_push($filterGrid, array(
    	    'columnName'=> 'P_NRLISTA',
    	    'operator'=> '=',
    	    'value' => $nrlista
	    ));
	    switch ($conditions[0]['value']) {
	        case 'A':
	            $dataSourceName = 'ger04102_grid_alterar';
	            break;
            case 'S':
	            $dataSourceName = 'ger04102_grid_sobrepor';
	            break;
            case 'I':
	            $dataSourceName = 'ger04102_grid_incluir';
	            break;
	    }
    	
    	$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $filterGrid);
    	$newDataSet = new DataSet('ger04102_grid', $dataSet->getRows());
        $response->addDataSet($newDataSet);
        $response->addDataSet(new DataSet("nrlista", array($nrlista)));
    }
    
    public function retornaListaServico($filterServ){
        if($filterServ[4]['value'] == 'T'){
    		$dataSet = $this->crudService->getFilteredDataSet('ger04102_filtro_servico', array_slice($filterServ, 0, 4));
    		$dados = $dataSet->getRows();
    		return $dados;
        }else{
            return $filterServ[4]['value'];
        }
    }
     
    //insere lista no banco
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //apaga lista do banco
    public function deleteLista($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista
        );
        
        $sql = "DELETE FROM LISTA WHERE NRLISTA =:NRLISTA";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    public function salvaServicos($request, $response){
        $row = $request->getRow();
        switch ($row['OPERACAO']) {
            case 'A':
                $this->salvaAlteracao($request, $response);
                break;
            case 'I':
                $this->salvaInclusao($request, $response);
                break;
            case 'S':
                $this->salvaSobreposicao($request, $response);
                break;
        }
    }
    
    public function apagaListas($request, $response){
        $row = $request->getRow();
        foreach($row['NRLISTAS'] as $nrlista){
            $this->deleteLista($nrlista);
        }
    }
    
    public function excluiCustoPadrao($request, $response){
        $rows = $request->getDataSet()->getRows();
        try{
            foreach($rows as $row){
                $this->servicoService->deletaCustoPadrao($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTREFCUSPADR'], $row['NRORG']);
            }
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function salvaAlteracao($request, $response){
        $rowToSave = $request->getRow();
        $rows = $rowToSave['ROWS'];
        foreach($rows as &$row){
            $this->connection->beginTransaction();
            try{
                $row['__is_new'] = false;
                $row['changed'] = true;
                $row['OLD_VRCUSTPADR'] = $row['VRCUSTPADR'];
                $row['VRCUSTPADR'] = $row['NOVO_VALOR'];
                $dataSet = new DataSet("custpadr", array($row));
                $request = new DTO\Request\DataSet($dataSet, "", "", "");
                $rts = $this->crudService->getRowsToSave($request, "custpadr");
                $this->crudService->save("custpadr", $rts);
                
                $conditions = array(
                    array(
                        "columnName" => "P_CDFILIAL",
                        "operator"   => '=',
                        "value"      => $row['CDFILIAL']
                    ),
                    array(
                        "columnName" => "P_CDSERVFILI",
                        "operator"   => '=',
                        "value"      => $row['CDSERVFILI']
                    ),
                    array(
                        "columnName" => "DTREFCUSPADR",
                        "operator"   => '=',
                        "value"      => $row['DTREFCUSPADR']
                    )
                );
                $dataSet = $this->crudService->getFilteredDataSet("ger04102_custos_servico", $conditions); 
                $this->realizaRateio($dataSet->getRows(), $row);
                $this->connection->commit();
            }catch(\Exception $e){
                $this->connection->rollBack(); 
                throw $e;
            }
        }
    }
    
    public function salvaInclusao($request, $response){
        $rowToSave = $request->getRow();
        $rows = $rowToSave['ROWS'];
        foreach($rows as &$row){
            $this->connection->beginTransaction();
            try{
                $row['DTINICIAL'] = $rowToSave['DTINICIAL'];
                $row['DTFINAL'] = $rowToSave['DTFINAL'];
                $servicos = $this->servicoService->retornaCustosComData($row);
                foreach($servicos as $servico){
                    $servico['DTINICIAL'] = $rowToSave['DTINICIAL'];
                    $servico['__is_new'] = $servico['POSSUI_CUSTO'] == 'S' ? false : true;
                    $servico['changed'] = true;
                    $servico['OLD_VRCUSTPADR'] = $servico['VRCUSTPADR'];
                    $servico['NOVO_VALOR'] = $row['NOVO_VALOR'];
                    $servico['VRCUSTPADR'] = $servico['NOVO_VALOR'];
                    $dataSet = new DataSet("custpadr", array($servico));
                    $request = new DTO\Request\DataSet($dataSet, "", "", "");
                    $rts = $this->crudService->getRowsToSave($request, "custpadr");
                    $this->crudService->save("custpadr", $rts);
                    
                    if($servico['POSSUI_CUSTO'] == 'S'){
                        $conditions = array(
                            array(
                                "columnName" => "P_CDFILIAL",
                                "operator"   => '=',
                                "value"      => $row['CDFILIAL']
                            ),
                            array(
                                "columnName" => "P_CDSERVFILI",
                                "operator"   => '=',
                                "value"      => $row['CDSERVFILI']
                            ),
                            array(
                                "columnName" => "DTREFCUSPADR",
                                "operator"   => '=',
                                "value"      => $servico['DTREFCUSPADR']
                            )
                        );
                        $dataSet = $this->crudService->getFilteredDataSet("ger04102_custos_servico", $conditions); 
                        $this->realizaRateio($dataSet->getRows(), $servico);
                    }else{
                        $this->servicoService->insereCustoPadrao($servico);
                    }
                }
                $this->connection->commit();
            }catch(\Exception $e){
                $this->connection->rollBack(); 
                throw $e;
            }
        }
    }
    
    public function salvaSobreposicao($request, $response){
        $rowToSave = $request->getRow();
        $rows = $rowToSave['ROWS'];
        foreach($rows as &$row){
            $this->connection->beginTransaction();
            try{
                $row['DTINICIAL'] = $rowToSave['DTINICIAL'];
                $row['__is_new'] = $row['EXIST_CUSTO'] == 'S' ? false : true;
                $row['changed'] = true;
                $row['OLD_VRCUSTPADR'] = $row['VRCUSTPADR'];
                $row['NOVO_VALOR'] = $row['NOVO_VALOR'];
                $row['VRCUSTPADR'] = $row['NOVO_VALOR'];
                $dataSet = new DataSet("custpadr", array($row));
                $request = new DTO\Request\DataSet($dataSet, "", "", "");
                $rts = $this->crudService->getRowsToSave($request, "custpadr");
                $this->crudService->save("custpadr", $rts);
                
                if($row['EXIST_CUSTO'] == 'S'){
                    $conditions = array(
                        array(
                            "columnName" => "P_CDFILIAL",
                            "operator"   => '=',
                            "value"      => $row['CDFILIAL']
                        ),
                        array(
                            "columnName" => "P_CDSERVFILI",
                            "operator"   => '=',
                            "value"      => $row['CDSERVFILI']
                        ),
                        array(
                            "columnName" => "DTREFCUSPADR",
                            "operator"   => '=',
                            "value"      => $row['DTREFCUSPADR']
                        )
                    );
                    $dataSet = $this->crudService->getFilteredDataSet("ger04102_custos_servico", $conditions); 
                    $this->realizaRateio($dataSet->getRows(), $row);
                }else{
                    $this->servicoService->insereCustoPadrao($row);
                }
                $this->connection->commit();
            }catch(\Exception $e){
                $this->connection->rollBack(); 
                throw $e;
            }
        }
    }
    
    public function realizaRateio($tiposDePrato, $custoPadrao){
        $length = count($tiposDePrato);
        $soma = 0;
        $diff;
        try{
            if($length > 0){
                foreach ($tiposDePrato as $index => &$tipo) {
                    if($custoPadrao['OLD_VRCUSTPADR'] == 0) {//total do custo
                        $fator = 1/$length;
                    } else {
                        $fator = $tipo['VRCUSTPADRTP'] / $custoPadrao['OLD_VRCUSTPADR'];
                    }
                    
                    $tipo['VRCUSTPADRTP'] = $fator*$custoPadrao['NOVO_VALOR'];
                    $soma += $tipo['VRCUSTPADRTP'];
                    if($soma > 0 && $soma != $custoPadrao['NOVO_VALOR'] && ($index === $length-1)){
                        $dif = $custoPadrao['NOVO_VALOR'] - $soma;
                        $tipo['VRCUSTPADRTP'] = round(($tipo['VRCUSTPADRTP'] + $dif),5);
                    }
                    $tipo['changed'] = true;
                    $tipo['__is_new'] = $tipo['NOVOCUSTO'] == 'S' ? true : false;
                    $dataSet = new DataSet("custpadrtp", array($tipo));
                    $request = new DTO\Request\DataSet($dataSet, "", "", "");
                    $rts = $this->crudService->getRowsToSave($request, "custpadrtp");
                    
                    $this->crudService->save("custpadrtp", $rts);
                }
            }
            return $tiposDePrato;
        }catch(\Exception $e){
            throw $e;
        }
    }
}