<?php
namespace Teknisa\PLA\Util;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Doctrine\ORM\EntityManager;

class Listas extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $entityManager;
    protected $connection;
    protected $crudService;
    protected $parameterbag;
    protected $newCode;
    protected $dataSourceManager;
    
    public function __construct($crudService, $parameterbag, $environment, $connection, $entityManager, $newCode, $dataSourceManager) {
        $this->crudService       = $crudService;
        $this->parameterbag      = $parameterbag;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->entityManager     = $entityManager;
        $this->newCode           = $newCode;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    //insere lista no banco
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista,
            "NRORG" => $this->environment->getNrorgTrab()
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //apaga lista do banco
    public function deleteLista($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista
        );
        
        $sql = "DELETE FROM LISTA WHERE NRLISTA =:NRLISTA";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //cria lista de valores recebendo string como parametro
    public function criarListaValores($valores){
        if(is_array($valores))
            $valores = implode('_', $valores);
        if($valores === 'T')
            return $valores;
        if(strlen($valores) <= 4000){
            return $valores;
        }else{
            $lista = explode('_',$valores);
            $lista = array_unique($lista);
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $key){
                    if($key != ''){
                        $this->insertLista($nrlista, $key);
                    }
                }
                return "_NRLISTA=".$nrlista;
            } catch(\Exception $e){
                throw $e;
            }
        }
    }
    
    public function criarListaValoresMultiplos($ArrayAssociativoDeValores){
        $resultado = array();
        foreach($ArrayAssociativoDeValores as $key => $value){
            $resultado[$key] = $this->criarListaValores($value);
        }
        return $resultado;
    }
    
    //apaga lista de valores recebendo string como parametro
    public function apagarListaValores($valores){
        try{
            if(substr($valores, 0,9) == "_NRLISTA="){
                $nrlista = substr($valores, 9);
                $this->deleteLista($nrlista);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
    public function retornaListaValores(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		$row = $this->criarListaValoresMultiplos($row);
		$response->addDataSet(new DataSet("listaValores", $row));
    }
    
    /***
     * Cria uma lista temporária com as filiais e serviços selecionadas
     * $cdGrupFili => vazio ou um código de agrupamento
     * $filiais => array de string, string concatenada ou "T"
     * $grupServ => array de string, string concatenada ou "T"
     * servicos => array  de array associativo ['CDFILIAL' E 'CDSERVFILI'] ou "T"
     * $vdServAtiv => bool indica se considera apenas serviços habilitados para requisição, retiradas
     * $vdDescentr => bool indica se considera apenas serviços que dependem de liberação
     * $vdServNaoHosp => bool indica se deve desconsiderar serviços hospitalares
     */
    public function retornaListaFilialServico($cdGrupFili, $filiais, $grupServ, $servicos, $vdServAtiv, $vdDescentr, $vdServNaoHosp){
        try{
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            $grupServ = $this->criarListaValores($grupServ);
            $filiais = $this->criarListaValores($filiais);
            $this->connection->beginTransaction();            
            if ($servicos == "T") {
                $sql = "
                INSERT INTO LISTA (NRLISTA, CDLISTA)
                SELECT :NRLISTA, S.CDFILIAL||'_'||S.CDSERVFILI FROM SERVICO S
                 WHERE EXISTS (SELECT 1 FROM FILIOPER WHERE CDFILIAL = S.CDFILIAL AND CDOPERADOR = :CDOPERADOR)
                   AND ((:CDGRUPFILI IS NULL) OR EXISTS(SELECT 1 FROM FILGRUPFILI FG WHERE FG.CDFILIAL = S.CDFILIAL AND FG.CDGRUPFILI = :CDGRUPFILI))
                   AND VALOR_NA_LISTA(S.CDFILIAL, :FILIAIS) > 0
                ";
                $prepared = $this->connection->prepare($sql);
                $params = [
                    "NRLISTA" => $nrlista,
                    "CDOPERADOR" => $this->environment->getCdOperador(),
                    "CDGRUPFILI" => $cdGrupFili,
                    "FILIAIS" => $filiais
                ];
                $prepared->execute($params);
            } else {
                $sql = "
                INSERT INTO LISTA (NRLISTA, CDLISTA)
                SELECT :NRLISTA, S.CDFILIAL||'_'||S.CDSERVFILI FROM SERVICO S
                 WHERE EXISTS (SELECT 1 FROM FILIOPER WHERE CDFILIAL = S.CDFILIAL AND CDOPERADOR = :CDOPERADOR)
                   AND ((:CDGRUPFILI IS NULL) OR EXISTS(SELECT 1 FROM FILGRUPFILI FG WHERE FG.CDFILIAL = S.CDFILIAL AND FG.CDGRUPFILI = :CDGRUPFILI))
                   AND S.CDFILIAL = :CDFILIAL
                   AND S.CDSERVFILI = :CDSERVFILI
                ";
                $prepared = $this->connection->prepare($sql);
                
                foreach($servicos as $serv){
                    $params = [
                        "NRLISTA" => $nrlista,
                        "CDOPERADOR" => $this->environment->getCdOperador(),
                        "CDGRUPFILI" => $cdGrupFili,
                        "CDFILIAL" => $serv['CDFILIAL'],
                        "CDSERVFILI" => $serv['CDSERVFILI']
                    ];
                    $prepared->execute($params);
                }
            }
            if($grupServ != "T"){
                $sql = "
                DELETE FROM LISTA L
                 WHERE L.NRLISTA = :NRLISTA
                   AND NOT EXISTS (SELECT 1 FROM SERVICO S 
                                    WHERE S.CDFILIAL = SUBSTR(L.CDLISTA, 1, 4)
                                      AND S.CDSERVFILI = SUBSTR(L.CDLISTA, 6, 10)
                                      AND VALOR_NA_LISTA(S.CDGRUPSERV, :GRUPSERV) > 0) 
                ";
                $prepared = $this->connection->prepare($sql);
                $params = [
                    "NRLISTA" => $nrlista,
                    "GRUPSERV" => $grupServ
                ];
                $prepared->execute($params);        
            }
            
            //Somente serviços ativos (habilitado para requisição, retirada) na lista
            if($vdServAtiv){
                $sql = "
                DELETE FROM LISTA L
                 WHERE L.NRLISTA = :NRLISTA
                   AND NOT EXISTS (SELECT 1 FROM SERVICO S 
                                    WHERE S.CDFILIAL = SUBSTR(L.CDLISTA, 1, 4)
                                      AND S.CDSERVFILI = SUBSTR(L.CDLISTA, 6, 10)
                                      AND S.IDSERVATIV <> 'N')
                ";
                $prepared = $this->connection->prepare($sql);
                $params = [
                    "NRLISTA" => $nrlista
                ];
                $prepared->execute($params);
            }
            
            //Somente serviços descentralizados na lista
            if($vdDescentr){
                $sql = "
                DELETE FROM LISTA L
                 WHERE L.NRLISTA = :NRLISTA
                   AND NOT EXISTS (SELECT 1 FROM SERVICO S 
                                    WHERE S.CDFILIAL = SUBSTR(L.CDLISTA, 1, 4)
                                      AND S.CDSERVFILI = SUBSTR(L.CDLISTA, 6, 10)
                                      AND S.IDCARDDESCENTR = 'S')
                ";
                $prepared = $this->connection->prepare($sql);
                $params = [
                    "NRLISTA" => $nrlista
                ];
                $prepared->execute($params);
            }
            
            //Somente serviços não hospitalares na lista
            if($vdServNaoHosp){
                $sql = "
                DELETE FROM LISTA L
                 WHERE L.NRLISTA = :NRLISTA
                   AND NOT EXISTS (SELECT 1 FROM SERVICO S 
                                    WHERE S.CDFILIAL = SUBSTR(L.CDLISTA, 1, 4)
                                      AND S.CDSERVFILI = SUBSTR(L.CDLISTA, 6, 10)
                                      AND ( ( S.IDTPSERVHOSP IS NULL ) OR ( S.IDTPSERVHOSP = '0' ) ) )
                ";
                $prepared = $this->connection->prepare($sql);
                $params = [
                    "NRLISTA" => $nrlista
                ];
                $prepared->execute($params);
            }
            $this->connection->commit();
            
            return $nrlista;
        }catch(\Exception $e){
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function apagaLista(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $this->deleteLista($row['NRLISTA']);
    }
    
    //cria lista de valores recebendo string como parametro
    public function criarListaDoArray($arrayValores){
        if(!is_array($arrayValores))
            return null;
        $lista = array_unique($arrayValores);
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        try{
            foreach($lista as $key){
                if($key != ''){
                    $this->insertLista($nrlista, $key);
                }
            }
            return $nrlista;
        } catch(\Exception $e){
            throw $e;
        }
        return $nrlista;
    }
    
}